<div ng:controller="ScriptWidgetController">
    <div class="widget-header" translate>
        Script
    </div>
    <div class="pull-right">
        <a ng:click="run(widget.config)" class="btn btn-default">
            <i class="fa fa-play"></i> <span translate>Run</span>
        </a>
    </div>
    <div class="widget-value">
        {{widget.config.name}}
    </div>
</div>
