{"": "", "stdin data": "", "Terminal was closed": "", "Run arbitrary scripts": "", "Run": "", "Name": "", "Script has finished": "", "Copy terminal content": "", "Could not launch the script": "", "Script": "", "Terminate": "", "Terminal": "", "Terminals": "", "Done": "", "Open shell terminals": "", "Run a command...": "", "Open a new terminal": "", "Starting the script": "", "Script has failed": ""}